package com.WebFluxReactiveMongo.Reactor.Controller;

import com.WebFluxReactiveMongo.Reactor.Dto.StudentDto;
import com.WebFluxReactiveMongo.Reactor.Service.StudentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;


@RestController
@RequestMapping("/students")
@Slf4j
public class StudentController {

    @Autowired
    private StudentService studentService;
    @GetMapping
    public Flux<StudentDto> getStudents(){
        return studentService.getStudents();
    }

    /*@GetMapping
    public Flux<Student> getStudentsTemp(){
        log.info("inside controller=========");
        return studentService.getStudentsWithTemp();
    }*/

    @GetMapping("/{id}")
    public Mono<StudentDto> studentById(@PathVariable String id){
        return studentService.getStudentById(id);
    }

    @GetMapping("/student-fee-range")
    public Flux<StudentDto> getStudentFeeRange(@RequestParam("min") double min, @RequestParam("max") double max){
        return studentService.getStudentInFee(min,max);
    }

    @PostMapping
    public Mono<StudentDto> saveStudent(@RequestBody Mono<StudentDto> studentDtoMono){
        return studentService.saveStudent(studentDtoMono);
    }

    @PutMapping("/update/{id}")
    public Mono<StudentDto> updateStudent(@RequestBody Mono<StudentDto> studentDtoMono, @PathVariable String id){
        return studentService.updateStudent(studentDtoMono,id);
    }

    @DeleteMapping("/{id}")
    public Mono<Void> deleteStudent(@PathVariable String id){
        return studentService.deleteStudent(id);
    }

}
