package com.WebFluxReactiveMongo.Reactor.Service;

import com.WebFluxReactiveMongo.Reactor.Entity.Student;
import com.WebFluxReactiveMongo.Reactor.Util.ApplicaitonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Range;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import com.WebFluxReactiveMongo.Reactor.Repository.StudentRepository;
import com.WebFluxReactiveMongo.Reactor.Dto.StudentDto;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
@Slf4j
public class StudentService {

    @Autowired
    private StudentRepository studentRepository;

    @Autowired
    private ReactiveMongoTemplate reactiveMongoTemplate;

    public StudentService(ReactiveMongoTemplate reactiveMongoTemplate){
        this.reactiveMongoTemplate = reactiveMongoTemplate;
    }

    public Flux<Student> getStudentsWithTemp(){
        Query query = new Query()
                .addCriteria(Criteria.where("name").is("raju"))
                .with(Sort.by(Sort.Order.desc("fee")))
                .limit(3);
        log.info("query======="+query);
        return reactiveMongoTemplate.find(query, Student.class);
    }

    public Flux<StudentDto> getStudents(){
        return studentRepository.findAll().map(ApplicaitonUtil::entityToDto);
    }

    public Mono<StudentDto> getStudentById(String id){
        return studentRepository.findById(id).map(ApplicaitonUtil::entityToDto);
    }

    public Flux<StudentDto> getStudentInFee(double min, double max){
        return studentRepository.findByFeeBetween(Range.closed(min,max));
    }

    public Flux<StudentDto> getStudentInId(int min,int max){
        return studentRepository.findByIdBetween(Range.closed(min,max));
    }

    public Mono<StudentDto> saveStudent(Mono<StudentDto> studentDtoMono){
        return studentDtoMono.map(ApplicaitonUtil::dtoToEntity).
                flatMap(studentRepository::insert).
                map(ApplicaitonUtil::entityToDto);
    }


	  public Mono<StudentDto> updateStudent(Mono<StudentDto> studentDtoMono, String id){
        return studentRepository.findById(id).
              flatMap(p->studentDtoMono.map(ApplicaitonUtil::dtoToEntity)
                      .doOnNext(e->e.setId(id))) .flatMap(studentRepository::save)
              .map(ApplicaitonUtil::entityToDto);
    }


    public Mono<Void> deleteStudent(String id){
        return studentRepository.deleteById(id);
    }
}
