package com.WebFluxReactiveMongo.Reactor.kafka;

import com.WebFluxReactiveMongo.Reactor.Dto.StudentDto;
import com.WebFluxReactiveMongo.Reactor.Repository.StudentRepository;
import com.WebFluxReactiveMongo.Reactor.Service.StudentService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.support.serializer.JsonSerializer;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.kafka.sender.KafkaSender;
import reactor.kafka.sender.SenderOptions;
import reactor.kafka.sender.SenderRecord;

import org.apache.kafka.common.serialization.StringSerializer;
import org.apache.kafka.common.serialization.IntegerSerializer;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

@Slf4j
public class SampleProducer {

    private static final String BOOTSTRAP_SERVERS = "localhost:9092";
    private static final String TOPIC = "TestTopic1";

    @Autowired
    private StudentService studentService;

    @Autowired
    private StudentRepository studentRepository;

    private final KafkaSender<String, JsonNode> sender;

    public SampleProducer(String bootstrapServers) {

        Map<String, Object> props = new HashMap<>();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        props.put(ProducerConfig.CLIENT_ID_CONFIG, "sample-producer");
        props.put(ProducerConfig.ACKS_CONFIG, "all");
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, JsonSerializer.class);
        SenderOptions<String, JsonNode> senderOptions = SenderOptions.create(props);

        sender = KafkaSender.create(senderOptions);
    }

    public void sendMessages(String topic, int count, CountDownLatch latch) throws InterruptedException {
        ObjectMapper objectMapper = new ObjectMapper();
        //Flux<StudentDto> studentDto = Flux.just(new StudentDto("106","AADHI","DIYA",50000),
                //new StudentDto("107","VASU","KIYA",45000));
        sender.<Integer>send(Flux.range(1, 1)
                        //.map(i -> SenderRecord.create(new ProducerRecord<>(topic, i, "Message_" + i), i)))
                .map(i -> {
                    JsonNode jsonValue = null;
                    try {
                        Mono<StudentDto> studentDtoMono=Mono.just(new StudentDto("5855","Kafka Producer message","Nisum School",60000));
                        String value = objectMapper.writeValueAsString(studentDtoMono);
                        jsonValue = objectMapper.readTree(value);

                    } catch (JsonProcessingException e) {
                        e.printStackTrace();
                    }
                    log.info("Message: {}, sent successfully", jsonValue);
                    return SenderRecord.create(new ProducerRecord<>(topic, "Key_" + UUID.randomUUID(), jsonValue), i);
                }))
                .doOnError(e -> log.error("Send failed", e))
                            .doOnNext(r -> log.debug("Message #{} result: {}", r.correlationMetadata(), r.recordMetadata()))
                            .subscribe();
                    latch.countDown();
                }

    public void close() {
        sender.close();
    }

    public static void main(String[] args) throws Exception {
        int count = 20;
        CountDownLatch latch = new CountDownLatch(count);
        SampleProducer producer = new SampleProducer(BOOTSTRAP_SERVERS);
        producer.sendMessages(TOPIC, count, latch);
        latch.await(10, TimeUnit.SECONDS);
        producer.close();
    }
}
