package com.WebFluxReactiveMongo.Reactor;

import com.WebFluxReactiveMongo.Reactor.Controller.StudentController;
import com.WebFluxReactiveMongo.Reactor.Dto.StudentDto;
import com.WebFluxReactiveMongo.Reactor.Service.StudentService;
import org.junit.Test;
//import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.when;
import static org.mockito.ArgumentMatchers.any;

@RunWith(SpringRunner.class)
@WebFluxTest(StudentController.class)
public class SpringReactiveMangoApplicationTests {

	@Autowired
	private WebTestClient webTestClient;

	@MockBean
	private StudentService studentService;

	@Test
	public void addStudentTest(){
		Mono<StudentDto> studentDtoMono = Mono.just(new StudentDto("101","RAJU","CJJC",7000));
		when(studentService.saveStudent(studentDtoMono)).thenReturn(studentDtoMono);
		webTestClient.post().uri("/students").body(Mono.just(studentDtoMono), StudentDto.class)
				.exchange().expectStatus().isOk(); //200
	}

	@Test
	public void getStudentsTest(){
		Flux<StudentDto> studentDtoFlux = Flux.just(new StudentDto("102","RAVI","GDC",1500),
				new StudentDto("103","RAMU","KRDC",5500));
		when(studentService.getStudents()).thenReturn(studentDtoFlux);
		Flux<StudentDto> responseBody = webTestClient.get().uri("/students")
				.exchange().expectStatus().isOk()
				.returnResult(StudentDto.class)
				.getResponseBody();

		StepVerifier.create(responseBody)
				.expectSubscription()
				.expectNext(new StudentDto("102","RAVI","GDC",1500))
				.expectNext(new StudentDto("103","RAMU","KRDC",5500))
				.verifyComplete();
	}

	@Test
	public void getStudentTest(){
		Mono<StudentDto> studentDtoMono = Mono.just(new StudentDto("102","RAVI","GDC",1500));
		when(studentService.getStudentById(any())).thenReturn(studentDtoMono);
		Flux<StudentDto> responseBody = webTestClient.get().uri("/students/102")
				.exchange().expectStatus().isOk()
				.returnResult(StudentDto.class)
				.getResponseBody();

		StepVerifier.create(responseBody)
				.expectSubscription()
				.expectNextMatches(p->p.getName().equals("RAVI"))
				.verifyComplete();
	}

	@Test
	public void updateStudentTest(){
		Mono<StudentDto> studentDtoMono = Mono.just(new StudentDto("102","RAVI","GDC",1500));
		when(studentService.updateStudent(studentDtoMono,"102")).thenReturn(studentDtoMono);
		webTestClient.put().uri("/students/update/102")
				.exchange().expectStatus().isOk();
	}

	@Test
	public void deleteStudentTest(){
		given(studentService.deleteStudent("102")).willReturn(Mono.empty());
		webTestClient.delete().uri("/students/102")
				.exchange().expectStatus().isOk();
	}

}
